@@ -1,29 +1,20 @@
 /*
-    Copyright 2015 by Gregor Mi <codestruct@posteo.org>
+    SPDX-FileCopyrightText: 2015 Gregor Mi <codestruct@posteo.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+    SPDX-License-Identifier: LGPL-2.1-or-later
 */
 
 #ifndef KMORETOOLS_H
 #define KMORETOOLS_H
 
+#include <QMenu>
 #include <QString>
 #include <QUrl>
-#include <QMenu>
 
 #include <KService>
 
+#include <memory>
+
 #include "knewstuff_export.h"
 
 class KMoreToolsService;
@@ -45,7 +36,7 @@ class KMoreToolsPrivate;
  *
  * Introduction
  * ------------
- * KMoreTools helps to to build user-configurable menus with tools which
+ * KMoreTools helps to build user-configurable menus with tools which
  * might not installed yet. These tools may also take URL arguments supplied
  * by the application.
  *
@@ -58,7 +49,7 @@ class KMoreToolsPrivate;
  * - Provide an API to define external applications for a given context.
  * - If a defined application is not installed (yet) the application is (optionally)
  *    still presented to the user with a hint that it is not installed and a link
- *    to the homepage (later with integration to package managment).
+ *    to the homepage (later with integration to package management).
  *    This increases the discoverability of useful applications the user never
  *    heard about yet.
  * - In case of many applications for a given context, it provides a GUI to the
@@ -118,7 +109,7 @@ class KMoreToolsPrivate;
  *
  * ### KSnapshot's Send To... menu
  *
- * Last updated: 2015-04-17, uncommited demo, source code:
+ * Last updated: 2015-04-17, uncommitted demo, source code:
  * src/kde/kdegraphics/ksnapshot/ksnapshotsendtoactions.cpp
  *
  * Note, that the last item in the 'More' menu in the following screenshot was
@@ -128,13 +119,13 @@ class KMoreToolsPrivate;
  *
  * ### Dolphins's Space info menu
  *
- * Last updated: 2015-04-17, uncommited demo, source code: src/kde/applications/dolphin/src/statusbar/spaceinfotoolsmenu.cpp
+ * Last updated: 2015-04-17, uncommitted demo, source code: src/kde/applications/dolphin/src/statusbar/spaceinfotoolsmenu.cpp
  *
  * \image html kmoretools-dolphin-spaceinfo-1.png "Space info menu" width=100px
  *
  * ### Kate's Project plugin git menu
  *
- * Last updated: 2015-03-25, uncommited demo, source code:
+ * Last updated: 2015-03-25, uncommitted demo, source code:
  * src/kde/applications/kate/addons/project/kateprojecttreeviewcontextmenu.cpp
  *
  * \image html kmoretools-kate-project-1-all-installed.png "All git tools installed" width=100px
@@ -163,7 +154,7 @@ class KMoreToolsPrivate;
  * ----------
  * - question: KMoreTools::registerServiceByDesktopEntryName():
  *      - warn if service is not of Type=Application (KService::isApplication()) or just leave it?
-  * Add support for package managers to install software (e.g. muon discover)
+ * Add support for package managers to install software (e.g. muon discover)
  * - maybe: kmt-desktopfiles: add a config file that can configure the homepage URLs
  *   and e.g. the package name if needed for package manager support
  *
@@ -178,8 +169,7 @@ public:
     /**
      * Specify how should be determined if a service is installed or not
      */
-    enum ServiceLocatingMode
-    {
+    enum ServiceLocatingMode {
         /**
          * by existence of desktop file (discoverable by KService)
          */
@@ -189,14 +179,13 @@ public:
          * by existence of executable defined in the TryExec or Exec line of
          * the provided kmt-desktopfile
          */
-        ServiceLocatingMode_ByProvidedExecLine
+        ServiceLocatingMode_ByProvidedExecLine,
     };
 
     /**
      * Specify where a menu item be placed by default
      */
-    enum MenuSection
-    {
+    enum MenuSection {
         /**
          * The item is placed in the main section (default)
          */
@@ -205,32 +194,31 @@ public:
         /**
          * The item is placed in the "More" submenu.
          */
-        MenuSection_More
+        MenuSection_More,
     };
 
-//     /* *
-//      * todo/later: introduce when needed
-//      */
-//     enum NotInstalledSectionOption
-//     {
-//         /* *
-//          * default
-//          */
-//         NotInstalledSection_Show,
-//
-//         /* *
-//          * Even if there are non-installed apps the Not-Installed section will
-//          * not be shown
-//          */
-//         NotInstalledSection_Hide
-//     };
+    //     /* *
+    //      * todo/later: introduce when needed
+    //      */
+    //     enum NotInstalledSectionOption
+    //     {
+    //         /* *
+    //          * default
+    //          */
+    //         NotInstalledSection_Show,
+    //
+    //         /* *
+    //          * Even if there are non-installed apps the Not-Installed section will
+    //          * not be shown
+    //          */
+    //         NotInstalledSection_Hide
+    //     };
 
     /**
      * Specify if the Configure dialog be accessible from the menu
      * (via a "Configure..." menu item)
      */
-    enum ConfigureDialogAccessibleSetting
-    {
+    enum ConfigureDialogAccessibleSetting {
         /**
          * Always show the "Configure..." menu item
          * (default)
@@ -249,7 +237,7 @@ public:
          * Note, that the "Configure..." menu item still becomes visible when the
          * user holds the Ctrl key while opening the menu.
          */
-        ConfigureDialogAccessible_Defensive
+        ConfigureDialogAccessible_Defensive,
     };
 
 public:
@@ -260,7 +248,7 @@ public:
      * 2) the location where the kmt-desktopfiles should be installed because
      *    there they will be searched by default.
      *    If @p uniqueId contains slashes they will result in subdirectories.
-     *    The default location can be overriden by
+     *    The default location can be overridden by
      *    registerServiceByDesktopEntryName's kmtDesktopfileSubdir parameter.
      *    This is currently used in KMoreToolsPresets implementation to
      *    separate the kmt-desktopfiles location from the user's config section
@@ -288,10 +276,13 @@ public:
      *
      * To use it, you need to add \verbatim include(KDEInstallDirs) \endverbatim to your CMakeLists.txt.
      */
-    explicit KMoreTools(const QString& uniqueId);
+    explicit KMoreTools(const QString &uniqueId);
 
     ~KMoreTools();
 
+    KMoreTools(const KMoreTools &) = delete;
+    KMoreTools &operator=(const KMoreTools &) = delete;
+
     /**
      * Registers a service with KMoreTools.
      *
@@ -327,10 +318,9 @@ public:
      *         This kind of error must be fixed before you ship your application.
      *         This case is only used for unit tests.
      */
-    KMoreToolsService* registerServiceByDesktopEntryName(
-        const QString& desktopEntryName,
-        const QString& kmtDesktopfileSubdir = QString(),
-        ServiceLocatingMode serviceLocatingMode = ServiceLocatingMode_Default);
+    KMoreToolsService *registerServiceByDesktopEntryName(const QString &desktopEntryName,
+                                                         const QString &kmtDesktopfileSubdir = QString(),
+                                                         ServiceLocatingMode serviceLocatingMode = ServiceLocatingMode_Default);
 
     /**
      * @returns the interface to build the menu. It is a singleton instance
@@ -346,16 +336,10 @@ public:
      *
      * @sa KMoreToolsMenuBuilder::clear()
      */
-    KMoreToolsMenuBuilder* menuBuilder(const QString& userConfigPostfix = QString()) const;
+    KMoreToolsMenuBuilder *menuBuilder(const QString &userConfigPostfix = QString()) const;
 
 private:
-    /**
-     * No copy semantic => private and no implementation
-     */
-    KMoreTools(const KMoreTools&);
-
-private:
-    KMoreToolsPrivate* d;
+    const std::unique_ptr<KMoreToolsPrivate> d;
 };
 
 // --------------------------------------------------------------------------------------
@@ -380,6 +364,9 @@ class KNEWSTUFF_EXPORT KMoreToolsService
 public:
     ~KMoreToolsService();
 
+    KMoreToolsService(const KMoreToolsService &) = delete;
+    KMoreToolsService &operator=(const KMoreToolsService &) = delete;
+
     /**
      * @return the desktop entry name which the service is identified by and with which
      * it was registered (see registerServiceByDesktopEntryName).
@@ -429,7 +416,7 @@ public:
      * This way the user gets some information of how to install the
      * application.
      */
-    void setHomepageUrl(const QUrl& url);
+    void setHomepageUrl(const QUrl &url);
 
     /**
      * @see setMaxUrlArgCount()
@@ -439,7 +426,7 @@ public:
     /**
      * In KMoreToolsMenuFactory some minor magic is done. In the context of
      * connecting the action trigger signal we need to know the maximum number
-     * of URL arguments a given service can accept. Usaually a number between
+     * of URL arguments a given service can accept. Usually a number between
      * 0 and 1. Sometimes 2.
      * E.g. kdf must not be called with any positional argument.
      * E.g. gitg can be called with zero or one arguments.
@@ -465,7 +452,7 @@ public:
      * @sa KMoreToolsMenuItem::setInitialItemText
      * @sa KMoreToolsMenuBuilder::setInitialItemTextTemplate
      */
-    QString formatString(const QString& formatString) const;
+    QString formatString(const QString &formatString) const;
 
     /**
      * 1. Icon from installed desktop file
@@ -483,7 +470,7 @@ public:
      *
      * @see KService::setExec(...)
      */
-    void setExec(const QString& exec);
+    void setExec(const QString &exec);
 
     /**
      * Returns the associated appstream id that was previously set with setAppstreamId().
@@ -505,7 +492,7 @@ public:
      *
      * @since 5.48
      */
-    void setAppstreamId(const QString&);
+    void setAppstreamId(const QString &);
 
 private:
     /**
@@ -515,17 +502,18 @@ private:
      * @param installedService not nullptr if @p isInstalled is true
      * @param kmtDesktopfile not null if app-local kmt-desktopfile is found and valid
      */
-    KMoreToolsService(const QString& kmtDesktopfileSubdir,
-                      const QString& desktopEntryName,
-                      bool isInstalled, KService::Ptr installedService,
+    KMoreToolsService(const QString &kmtDesktopfileSubdir,
+                      const QString &desktopEntryName,
+                      bool isInstalled,
+                      KService::Ptr installedService,
                       KService::Ptr kmtDesktopfile);
 
     /**
      * No copy semantic => private and no implementation
      */
-    KMoreToolsService(const KMoreTools&);
+    KMoreToolsService(const KMoreTools &);
 
-    KMoreToolsServicePrivate* d;
+    const std::unique_ptr<KMoreToolsServicePrivate> d;
 };
 
 // --------------------------------------------------------------------------------------
@@ -553,6 +541,9 @@ class KNEWSTUFF_EXPORT KMoreToolsMenuBuilder
 public:
     ~KMoreToolsMenuBuilder();
 
+    KMoreToolsMenuBuilder(const KMoreToolsMenuBuilder &) = delete;
+    KMoreToolsMenuBuilder &operator=(const KMoreToolsMenuBuilder &) = delete;
+
     /**
      * Affects addMenuItem() if called before it.
      *
@@ -560,7 +551,7 @@ public:
      *
      * The default template text is "$GenericName".
      */
-    void setInitialItemTextTemplate(const QString& templateText);
+    void setInitialItemTextTemplate(const QString &templateText);
 
     /**
      * Adds a registered service (which can installed or not) to the menu.
@@ -581,8 +572,7 @@ public:
      *
      * @sa KMoreToolsMenuItem::action()
      */
-    KMoreToolsMenuItem* addMenuItem(KMoreToolsService* registeredService,
-                                    KMoreTools::MenuSection defaultLocation = KMoreTools::MenuSection_Main);
+    KMoreToolsMenuItem *addMenuItem(KMoreToolsService *registeredService, KMoreTools::MenuSection defaultLocation = KMoreTools::MenuSection_Main);
 
     /**
      * Adds an action to the menu which is created and managed by the caller.
@@ -600,8 +590,7 @@ public:
      *
      * @sa KMoreToolsMenuItem::action()
      */
-    KMoreToolsMenuItem* addMenuItem(QAction* action, const QString& itemId,
-                                    KMoreTools::MenuSection defaultLocation = KMoreTools::MenuSection_Main);
+    KMoreToolsMenuItem *addMenuItem(QAction *action, const QString &itemId, KMoreTools::MenuSection defaultLocation = KMoreTools::MenuSection_Main);
 
     /**
      * Clears all added menu items. This can be useful if the menuBuilder is reused more than once.
@@ -624,11 +613,9 @@ public:
      * Otherwise the pointer will set to nullptr.
      * This can be used to add some custom items to the @p menu.
      */
-    void buildByAppendingToMenu(
-        QMenu* menu,
-        KMoreTools::ConfigureDialogAccessibleSetting configureDialogAccessibleSetting
-        = KMoreTools::ConfigureDialogAccessible_Always,
-        QMenu** outMoreMenu = nullptr);
+    void buildByAppendingToMenu(QMenu *menu,
+                                KMoreTools::ConfigureDialogAccessibleSetting configureDialogAccessibleSetting = KMoreTools::ConfigureDialogAccessible_Always,
+                                QMenu **outMoreMenu = nullptr);
 
 private:
     /**
@@ -639,7 +626,7 @@ private:
     /**
      * for unit testing
      */
-    void showConfigDialog(const QString& title);
+    void showConfigDialog(const QString &title);
 
     /**
      * (needed because QMap needs a default ctor)
@@ -649,14 +636,14 @@ private:
     /**
      * internal usage
      */
-    KMoreToolsMenuBuilder(const QString& uniqueId, const QString& userConfigPostfix);
+    KMoreToolsMenuBuilder(const QString &uniqueId, const QString &userConfigPostfix);
 
     /**
      * No copy semantic => private and no implementation
      */
-    KMoreToolsMenuBuilder(const KMoreTools&);
+    KMoreToolsMenuBuilder(const KMoreTools &);
 
-    KMoreToolsMenuBuilderPrivate* d;
+    const std::unique_ptr<KMoreToolsMenuBuilderPrivate> d;
 };
 
 // --------------------------------------------------------------------------------------
@@ -682,6 +669,9 @@ class KNEWSTUFF_EXPORT KMoreToolsMenuItem
     friend class KMoreToolsMenuBuilder;
 
 public:
+    KMoreToolsMenuItem(const KMoreToolsMenuItem &) = delete;
+    KMoreToolsMenuItem &operator=(const KMoreToolsMenuItem &) = delete;
+
     /**
      * Auto-generated unique id that tries to be as stable as possible even if the
      * menu gets restructured after the user did some customization that was
@@ -702,7 +692,7 @@ public:
      * todo: make sure to show error if the id contains characters other than
      *       alphanumerica, dashes and underscores etc.
      */
-    void setId(const QString& id);
+    void setId(const QString &id);
 
     /**
      * @return the underlying KMoreToolsService instance,
@@ -710,7 +700,7 @@ public:
      * Or nullptr when KMoreToolsMenuBuilder::addMenuItem (with QAction* argument
      * was used).
      */
-    KMoreToolsService* registeredService() const;
+    KMoreToolsService *registeredService() const;
 
     /**
      * see KMoreToolsMenuBuilder::addMenuItem
@@ -735,7 +725,7 @@ public:
      * - action()
      * - You can use the static method KMoreToolsService::formatString here.
      */
-    void setInitialItemText(const QString& itemText);
+    void setInitialItemText(const QString &itemText);
 
     /**
      * Case 1
@@ -762,26 +752,20 @@ public:
      *
      * @see KMoreToolsService::isInstalled
      */
-    QAction* action() const;
+    QAction *action() const;
 
 private: // internal usage
     /**
      * Sets the initial item text.
      */
-    KMoreToolsMenuItem(KMoreToolsService* registeredService, KMoreTools::MenuSection defaultLocation, const QString& initialItemTextTemplate);
+    KMoreToolsMenuItem(KMoreToolsService *registeredService, KMoreTools::MenuSection defaultLocation, const QString &initialItemTextTemplate);
 
-    KMoreToolsMenuItem(QAction* action, const QString& itemId, KMoreTools::MenuSection defaultLocation);
+    KMoreToolsMenuItem(QAction *action, const QString &itemId, KMoreTools::MenuSection defaultLocation);
 
     ~KMoreToolsMenuItem();
 
 private:
-    /**
-     * No copy semantic => private and no implementation
-     */
-    KMoreToolsMenuItem(const KMoreTools&);
-
-private:
-    KMoreToolsMenuItemPrivate* d;
+    const std::unique_ptr<KMoreToolsMenuItemPrivate> d;
 };
 
 #endif // KMORETOOLS_H
